@@ -98,7 +98,11 @@ static int8_t _conn_iuup_amr_ft_2_rfci(struct mgcp_conn_rtp *conn_rtp, uint8_t f
 	uint8_t rfci_cnt = 0;
 	unsigned match_bytes = (unsigned)osmo_amr_bytes(ft);
 	struct osmo_iuup_rnl_prim *irp = conn_rtp->iuup.init_ind;
-	OSMO_ASSERT(irp);
+	if (!irp) {
+		/* No IuUP Initialization has occured on the IuUP side yet. Return error and drop the RTP data, until
+		 * the IuUP Initialization has configured the link. */
+		return -1;
+	}
 
 	/* TODO: cache this somehow */
 	for (i = 0; i < ARRAY_SIZE(irp->u.status.u.initialization.rfci); i++) {
@@ -260,6 +264,7 @@ static int bridge_iuup_to_rtp_peer(struct mgcp_conn_rtp *conn_rtp_src, struct mg
 	uint8_t *amr_data;
 	struct rtp_hdr *rtp_hdr;
 	struct amr_hdr *amr_hdr;
+	struct mgcp_rtp_codec *dst_codec;
 	int rc;
 
 	ft = osmo_amr_bytes_to_ft(msgb_l3len(msg));
@@ -270,9 +275,10 @@ static int bridge_iuup_to_rtp_peer(struct mgcp_conn_rtp *conn_rtp_src, struct mg
 		return ft;
 	}
 	msgb_pull_to_l3(msg);
-	LOGP(DLMGCP, LOGL_DEBUG, "Convert IuUP -> AMR: ft %d, len %d\n", ft, msgb_l3len(msg));
 
-	if (mgcp_codec_amr_is_octet_aligned(conn_rtp_dst->end.codec)) {
+	dst_codec = conn_rtp_dst->end.cset.codec;
+	if (mgcp_codec_amr_is_octet_aligned(dst_codec)) {
+		LOGP(DLMGCP, LOGL_DEBUG, "Convert IuUP -> AMR OA: ft %d, len %d\n", ft, msgb_length(msg));
 		amr_hdr = (struct amr_hdr *) msgb_push(msg, sizeof(struct amr_hdr));
 		amr_hdr->cmr = 15; /* no change */
 		amr_hdr->f = 0;
@@ -291,6 +297,7 @@ static int bridge_iuup_to_rtp_peer(struct mgcp_conn_rtp *conn_rtp_src, struct mg
 		amr_data[1] |= ((ft & 0x1) << 7) | (((!fqc) & 0x1) << 6);
 		amr_length = (osmo_amr_bits(ft) + 10 + 7) / 8;
 		msgb_trim(msg, amr_length);
+		LOGP(DLMGCP, LOGL_DEBUG, "Convert IuUP -> AMR BE: ft %d, len %zd\n", ft, amr_length);
 	}
 	rtp_hdr = (struct rtp_hdr *) msgb_push(msg, sizeof(*rtp_hdr));
 	*rtp_hdr = (struct rtp_hdr){
@@ -298,7 +305,7 @@ static int bridge_iuup_to_rtp_peer(struct mgcp_conn_rtp *conn_rtp_src, struct mg
 		.extension = 0,
 		.padding = 0,
 		.version = 0,
-		.payload_type = conn_rtp_dst->end.codec->payload_type,
+		.payload_type = dst_codec->payload_type,
 		.marker = 0,
 		.sequence = frame_nr,
 		.timestamp = 0,
@@ -306,7 +313,6 @@ static int bridge_iuup_to_rtp_peer(struct mgcp_conn_rtp *conn_rtp_src, struct mg
 	};
 
 	rc = mgcp_send(conn_rtp_dst->conn->endp, true, NULL, msg, conn_rtp_src, conn_rtp_dst);
-	msgb_free(msg);
 	return rc;
 }
 
@@ -342,8 +348,7 @@ static int _conn_iuup_rx_rnl_data(struct mgcp_conn_rtp *conn_rtp_src, struct osm
 		return bridge_iuup_to_iuup_peer(conn_rtp_src, conn_rtp_dst, irp);
 	case MGCP_RTP_DEFAULT:
 		return bridge_iuup_to_rtp_peer(conn_rtp_src, conn_rtp_dst, irp);
-	case MGCP_OSMUX_BSC:
-	case MGCP_OSMUX_BSC_NAT:
+	case MGCP_RTP_OSMUX:
 	default:
 		LOGPCONN(conn_rtp_src->conn, DRTP, LOGL_ERROR,
 			 "Forward of IuUP payload to RTP connection type %u not supported!\n",
@@ -465,7 +470,7 @@ static int mgcp_send_iuup(struct mgcp_endpoint *endp, struct msgb *msg,
 	struct rtp_hdr *hdr = (struct rtp_hdr *)msgb_data(msg);
 	int buflen = msgb_length(msg);
 	char *dest_name;
-	int len;
+	int rc;
 
 	OSMO_ASSERT(conn_src);
 	OSMO_ASSERT(conn_dst);
@@ -490,7 +495,7 @@ static int mgcp_send_iuup(struct mgcp_endpoint *endp, struct msgb *msg,
 			 "rtp_port:%u rtcp_port:%u\n",
 			 dest_name,
 			 osmo_sockaddr_ntop(&rtp_end->addr.u.sa, ipbuf),
-			 ntohs(rtp_end->rtp_port), ntohs(rtp_end->rtcp_port)
+			 osmo_sockaddr_port(&rtp_end->addr.u.sa), ntohs(rtp_end->rtcp_port)
 			);
 		return 0;
 	}
@@ -499,30 +504,28 @@ static int mgcp_send_iuup(struct mgcp_endpoint *endp, struct msgb *msg,
 	 * ignored by the receiver, but still it's useful for debug purposes
 	 * to set it. Moreover, it seems ip.access nano3g produces much worse
 	 * audio output on the air side if timestamp is not set properly. */
-	hdr->timestamp = osmo_htonl(mgcp_get_current_ts(rtp_end->codec->rate));
+	hdr->timestamp = osmo_htonl(mgcp_get_current_ts(rtp_end->cset.codec->rate));
 	hdr->sequence = osmo_htons(rtp_state->alt_rtp_tx_sequence);
 	hdr->ssrc = rtp_state->alt_rtp_tx_ssrc;
+	rtp_state->alt_rtp_tx_sequence++;
 
 	LOGPENDP(endp, DRTP, LOGL_DEBUG,
 		 "process/send IuUP to %s %s rtp_port:%u rtcp_port:%u\n",
 		 dest_name, osmo_sockaddr_ntop(&rtp_end->addr.u.sa, ipbuf),
-		 ntohs(rtp_end->rtp_port), ntohs(rtp_end->rtcp_port));
+		 osmo_sockaddr_port(&rtp_end->addr.u.sa), ntohs(rtp_end->rtcp_port));
 
 	/* Forward a copy of the RTP data to a debug ip/port */
-	forward_data_tap(rtp_end->rtp.fd, &conn_src->tap_out,
-		     msg);
+	forward_data_tap(rtp_end->rtp, &conn_src->tap_out, msg);
 
-	len = mgcp_udp_send(rtp_end->rtp.fd, &rtp_end->addr, rtp_end->rtp_port,
-			    (char *)hdr, buflen);
+	rc = mgcp_udp_send(rtp_end->rtp, &rtp_end->addr, (char *)hdr, buflen);
 
-	if (len <= 0)
-		return len;
+	if (rc < 0)
+		return rc;
 
 	rtpconn_rate_ctr_add(conn_dst, endp, RTP_PACKETS_TX_CTR, 1);
-	rtpconn_rate_ctr_add(conn_dst, endp, RTP_OCTETS_TX_CTR, len);
-	rtp_state->alt_rtp_tx_sequence++;
+	rtpconn_rate_ctr_add(conn_dst, endp, RTP_OCTETS_TX_CTR, buflen);
 
-	return len;
+	return 0;
 }
 
 /* Received TNL primitive from IuUP layer FSM, transmit it further down to the
@@ -543,13 +546,13 @@ static int _conn_iuup_transport_prim_cb(struct osmo_prim_hdr *oph, void *ctx)
 
 	msgb_pull_to_l2(msg);
 	rtph = (struct rtp_hdr *)msgb_push(msg, sizeof(*rtph));
-	/* TODO: fill rtph properly: */
+	/* rtph is further filled in mgcp_send_iuup() below. */
 	*rtph = (struct rtp_hdr){
 		.csrc_count = 0,
 		.extension = 0,
 		.padding = 0,
 		.version = 2,
-		.payload_type = conn_rtp_dst->end.codec->payload_type,
+		.payload_type = conn_rtp_dst->end.cset.codec->payload_type,
 		.marker = 0,
 		.sequence = 0,
 		.timestamp = 0,
@@ -618,9 +621,8 @@ int mgcp_conn_iuup_dispatch_rtp(struct msgb *msg)
 		force_output_enabled = true;
 		/* Fill in the peer address so that we can send Init-ACK back: */
 		prev_rem_addr = conn_rtp_src->end.addr;
-		prev_rem_rtp_port = conn_rtp_src->end.rtp_port;
+		prev_rem_rtp_port = osmo_sockaddr_port(&conn_rtp_src->end.addr.u.sa);
 		conn_rtp_src->end.addr = *mc->from_addr;
-		conn_rtp_src->end.rtp_port = htons(osmo_sockaddr_port(&mc->from_addr->u.sa));
 	}
 
 	rc = _conn_iuup_rtp_pl_up(conn_rtp_src, msg);
@@ -628,7 +630,7 @@ int mgcp_conn_iuup_dispatch_rtp(struct msgb *msg)
 	if (force_output_enabled) {
 		conn_rtp_src->end.output_enabled = prev_output_enabled;
 		conn_rtp_src->end.addr = prev_rem_addr;
-		conn_rtp_src->end.rtp_port = prev_rem_rtp_port;
+		osmo_sockaddr_set_port(&conn_rtp_src->end.addr.u.sa, prev_rem_rtp_port);
 	}
 
 	return rc;
@@ -638,13 +640,14 @@ free_ret:
 }
 
 /* Build IuUP RNL Data primitive from msg containing an incoming RTP pkt from
- * peer and send it down the IuUP layer towards the destination as IuUP/RTP: */
+ * peer and send it down the IuUP layer towards the destination as IuUP/RTP. Takes ownership of msg. */
 int mgcp_conn_iuup_send_rtp(struct mgcp_conn_rtp *conn_src_rtp, struct mgcp_conn_rtp *conn_dest_rtp, struct msgb *msg)
 {
 	struct osmo_iuup_rnl_prim *irp;
 	struct rtp_hdr *rtph;
 	int rc = -1;
 	int iuup_length = 0;
+	struct mgcp_rtp_codec *src_codec;
 	int8_t rfci;
 
 	/* Tx RNL-DATA.req */
@@ -657,20 +660,21 @@ int mgcp_conn_iuup_send_rtp(struct mgcp_conn_rtp *conn_src_rtp, struct mgcp_conn
 
 	/* TODO: CMR handling & multiple frames handling */
 
-	if (strcmp(conn_src_rtp->end.codec->subtype_name, "AMR") != 0) {
+	src_codec = conn_src_rtp->end.cset.codec;
+	if (strcmp(src_codec->subtype_name, "AMR") != 0) {
 		LOG_CONN_RTP(conn_src_rtp, LOGL_ERROR,
 			     "Bridge RTP=>IuUP: Bridging src codec %s to IuUP AMR not supported\n",
-			     conn_src_rtp->end.codec->subtype_name);
+			     src_codec->subtype_name);
 		goto free_ret;
 	}
-	if (mgcp_codec_amr_is_octet_aligned(conn_src_rtp->end.codec)) {
+	if (mgcp_codec_amr_is_octet_aligned(src_codec)) {
 		struct amr_hdr *amr_hdr = (struct amr_hdr *) msgb_data(msg);
 		if (msgb_length(msg) < (sizeof(*amr_hdr))) {
 			LOG_CONN_RTP(conn_src_rtp, LOGL_NOTICE,
 				     "Bridge RTP=>IuUP: too short for AMR OA hdr (%u)\n", msgb_length(msg));
 			goto free_ret;
 		}
-		if (amr_hdr->ft >= AMR_FT_MAX) {
+		if (!osmo_amr_ft_valid(amr_hdr->ft)) {
 			LOG_CONN_RTP(conn_src_rtp, LOGL_NOTICE, "Bridge RTP=>IuUP: wrong AMR OA ft=%u\n", amr_hdr->ft);
 			goto free_ret;
 		}
@@ -678,19 +682,22 @@ int mgcp_conn_iuup_send_rtp(struct mgcp_conn_rtp *conn_src_rtp, struct mgcp_conn
 			LOG_CONN_RTP(conn_dest_rtp, LOGL_NOTICE, "Bridge RTP=>IuUP: No RFCI found for AMR OA ft=%u\n", amr_hdr->ft);
 			goto free_ret;
 		}
-		irp->u.data.fqc = amr_hdr->q;
+		irp->u.data.fqc = amr_hdr->q ? IUUP_FQC_FRAME_GOOD : IUUP_FQC_FRAME_BAD;
 		irp->u.data.rfci = rfci;
 		msgb_pull(msg, 2);
+		LOGP(DLMGCP, LOGL_DEBUG, "Convert AMR OA -> IuUP: ft %d -> rfci %d len %d\n",
+		     amr_hdr->ft, rfci, msgb_length(msg));
 	} else {
 		uint8_t *amr_bwe_hdr = (uint8_t *) msgb_data(msg);
 		int8_t ft;
+		uint8_t q;
 		if (msgb_length(msg) < 2) {
 			LOG_CONN_RTP(conn_src_rtp, LOGL_NOTICE,
 				     "Bridge RTP=>IuUP: too short for AMR BE hdr (%u)\n", msgb_length(msg));
 			goto free_ret;
 		}
 		ft = ((amr_bwe_hdr[0] & 0x07) << 1) | ((amr_bwe_hdr[1] & 0x80) >> 7);
-		if (ft >= AMR_FT_MAX) {
+		if (!osmo_amr_ft_valid(ft)) {
 			LOG_CONN_RTP(conn_src_rtp, LOGL_NOTICE, "Bridge RTP=>IuUP: wrong AMR BE ft=%u\n", ft);
 			goto free_ret;
 		}
@@ -698,7 +705,8 @@ int mgcp_conn_iuup_send_rtp(struct mgcp_conn_rtp *conn_src_rtp, struct mgcp_conn
 			LOG_CONN_RTP(conn_dest_rtp, LOGL_NOTICE, "Bridge RTP=>IuUP: No RFCI found for AMR BE ft=%u\n", ft);
 			goto free_ret;
 		}
-		irp->u.data.fqc = ((amr_bwe_hdr[1] & 0x40) >> 6);
+		q = amr_bwe_hdr[1] & 0x40;
+		irp->u.data.fqc = q ? IUUP_FQC_FRAME_GOOD : IUUP_FQC_FRAME_BAD;
 		irp->u.data.rfci = rfci;
 		rc = iuup_length = osmo_amr_bwe_to_iuup(msgb_data(msg), msgb_length(msg));
 		if (rc < 0) {
@@ -706,6 +714,8 @@ int mgcp_conn_iuup_send_rtp(struct mgcp_conn_rtp *conn_src_rtp, struct mgcp_conn
 			return rc;
 		}
 		msgb_trim(msg, iuup_length);
+		LOGP(DLMGCP, LOGL_DEBUG, "Convert AMR BE -> IuUP: ft %d -> rfci %d len %d\n",
+		     ft, rfci, msgb_length(msg));
 	}
 
 	irp->oph.msg->l3h = msgb_put(irp->oph.msg, msgb_length(msg));
