@@ -85,12 +85,6 @@ static inline struct msgb *mgw_msgb_copy_c(void *ctx, struct msgb *msg, const ch
 
 static int rx_rtp(struct msgb *msg);
 
-bool mgcp_rtp_end_remote_addr_available(const struct mgcp_rtp_end *rtp_end)
-{
-	return (osmo_sockaddr_port(&rtp_end->addr.u.sa) != 0) &&
-	       (osmo_sockaddr_is_any(&rtp_end->addr) == 0);
-}
-
 /*! Determine the local rtp bind IP-address.
  *  \param[out] addr caller provided memory to store the resulting IP-Address.
  *  \param[in] endp mgcp endpoint, that holds a copy of the VTY parameters.
@@ -338,7 +332,7 @@ static int adjust_rtp_timestamp_offset(const struct mgcp_endpoint *endp,
 				 osmo_sockaddr_ntop(&addr->u.sa, ipbuf),
 				 osmo_sockaddr_port(&addr->u.sa));
 		} else {
-			tsdelta = rtp_end->codec->rate * 20 / 1000;
+			tsdelta = rtp_end->cset.codec->rate * 20 / 1000;
 			LOGPENDP(endp, DRTP, LOGL_NOTICE,
 				 "Fixed packet duration and last timestamp delta "
 				 "are not available, "
@@ -502,11 +496,11 @@ static int mgcp_patch_pt(struct mgcp_conn_rtp *conn_dst, struct msgb *msg)
 	}
 	rtp_hdr = (struct rtp_hdr *)msgb_data(msg);
 
-	if (!conn_dst->end.codec) {
+	if (!conn_dst->end.cset.codec) {
 		LOG_CONN_RTP(conn_dst, LOGL_NOTICE, "no codec set on destination connection!\n");
 		return -EINVAL;
 	}
-	rtp_hdr->payload_type = (uint8_t) conn_dst->end.codec->payload_type;
+	rtp_hdr->payload_type = (uint8_t) conn_dst->end.cset.codec->payload_type;
 
 	return 0;
 }
@@ -529,7 +523,8 @@ void mgcp_patch_and_count(const struct mgcp_endpoint *endp,
 	uint32_t timestamp, ssrc;
 	bool marker_bit;
 	struct rtp_hdr *rtp_hdr;
-	int payload = rtp_end->codec->payload_type;
+	struct mgcp_rtp_codec *codec = rtp_end->cset.codec;
+	int payload = codec->payload_type;
 	unsigned int len = msgb_length(msg);
 
 	if (len < sizeof(*rtp_hdr))
@@ -538,7 +533,7 @@ void mgcp_patch_and_count(const struct mgcp_endpoint *endp,
 	rtp_hdr = (struct rtp_hdr *)msgb_data(msg);
 	seq = ntohs(rtp_hdr->sequence);
 	timestamp = ntohl(rtp_hdr->timestamp);
-	arrival_time = mgcp_get_current_ts(rtp_end->codec->rate);
+	arrival_time = mgcp_get_current_ts(codec->rate);
 	ssrc = ntohl(rtp_hdr->ssrc);
 	marker_bit = !!rtp_hdr->marker;
 	transit = arrival_time - timestamp;
@@ -547,16 +542,16 @@ void mgcp_patch_and_count(const struct mgcp_endpoint *endp,
 
 	if (!state->initialized) {
 		state->initialized = 1;
+		state->packet_duration = mgcp_rtp_packet_duration(endp, rtp_end);
 		state->in_stream.last_seq = seq - 1;
-		state->in_stream.ssrc = state->patch.orig_ssrc = ssrc;
+		state->in_stream.ssrc = ssrc;
 		state->in_stream.last_tsdelta = 0;
-		state->packet_duration =
-		    mgcp_rtp_packet_duration(endp, rtp_end);
 		state->out_stream.last_seq = seq - 1;
-		state->out_stream.ssrc = state->patch.orig_ssrc = ssrc;
 		state->out_stream.last_tsdelta = 0;
 		state->out_stream.last_timestamp = timestamp;
 		state->out_stream.ssrc = ssrc - 1;	/* force output SSRC change */
+		state->patch.orig_ssrc = ssrc;
+		state->patch.patch_ssrc = rtp_end->force_constant_ssrc;
 		LOGPENDP(endp, DRTP, LOGL_INFO,
 			 "initializing stream, SSRC: %u timestamp: %u "
 			 "pkt-duration: %d, from %s:%d\n",
@@ -566,7 +561,7 @@ void mgcp_patch_and_count(const struct mgcp_endpoint *endp,
 			 osmo_sockaddr_port(&addr->u.sa));
 		if (state->packet_duration == 0) {
 			state->packet_duration =
-			    rtp_end->codec->rate * 20 / 1000;
+			    codec->rate * 20 / 1000;
 			LOGPENDP(endp, DRTP, LOGL_NOTICE,
 				 "fixed packet duration is not available, "
 				 "using fixed 20ms instead: %d from %s:%d\n",
@@ -583,7 +578,7 @@ void mgcp_patch_and_count(const struct mgcp_endpoint *endp,
 			 osmo_sockaddr_port(&addr->u.sa));
 
 		state->in_stream.ssrc = ssrc;
-		if (rtp_end->force_constant_ssrc) {
+		if (state->patch.patch_ssrc) {
 			int16_t delta_seq;
 
 			/* Always increment seqno by 1 */
@@ -599,10 +594,7 @@ void mgcp_patch_and_count(const struct mgcp_endpoint *endp,
 			adjust_rtp_timestamp_offset(endp, state, rtp_end, addr,
 						    delta_seq, timestamp, marker_bit);
 
-			state->patch.patch_ssrc = true;
 			ssrc = state->patch.orig_ssrc;
-			if (rtp_end->force_constant_ssrc != -1)
-				rtp_end->force_constant_ssrc -= 1;
 
 			LOGPENDP(endp, DRTP, LOGL_NOTICE,
 				 "SSRC patching enabled, SSRC: %u "
@@ -822,8 +814,8 @@ static void gen_rtp_header(struct msgb *msg, struct mgcp_rtp_end *rtp_end,
 		return;
 
 	hdr->version = 2;
-	hdr->payload_type = rtp_end->codec->payload_type;
-	hdr->timestamp = osmo_htonl(mgcp_get_current_ts(rtp_end->codec->rate));
+	hdr->payload_type = rtp_end->cset.codec->payload_type;
+	hdr->timestamp = osmo_htonl(mgcp_get_current_ts(rtp_end->cset.codec->rate));
 	hdr->sequence = osmo_htons(state->alt_rtp_tx_sequence);
 	hdr->ssrc = state->alt_rtp_tx_ssrc;
 }
@@ -1189,6 +1181,8 @@ int mgcp_send(struct mgcp_endpoint *endp, int is_rtp, struct osmo_sockaddr *addr
 			 osmo_sockaddr_port(&rtp_end->addr.u.sa), ntohs(rtp_end->rtcp_port)
 		    );
 	} else if (is_rtp) {
+		struct mgcp_rtp_codec *src_codec;
+		struct mgcp_rtp_codec *dst_codec;
 		/* Make sure we have a valid RTP header, in cases where no RTP
 		 * header is present, we will generate one. */
 		gen_rtp_header(msg, rtp_end, rtp_state);
@@ -1204,19 +1198,21 @@ int mgcp_send(struct mgcp_endpoint *endp, int is_rtp, struct osmo_sockaddr *addr
 		if (addr)
 			mgcp_patch_and_count(endp, rtp_state, rtp_end, addr, msg);
 
+		src_codec = conn_src->end.cset.codec;
+		dst_codec = conn_dst->end.cset.codec;
 		if (mgcp_conn_rtp_is_iuup(conn_dst) || mgcp_conn_rtp_is_iuup(conn_src)) {
 			/* the iuup code will correctly transform to the correct AMR mode */
-		} else if (mgcp_codec_amr_align_mode_is_indicated(conn_dst->end.codec)) {
-			rc = amr_oa_bwe_convert(endp, msg, conn_dst->end.codec->param.amr_octet_aligned);
+		} else if (mgcp_codec_amr_align_mode_is_indicated(dst_codec)) {
+			rc = amr_oa_bwe_convert(endp, msg, dst_codec->param.amr_octet_aligned);
 			if (rc < 0) {
 				LOGPENDP(endp, DRTP, LOGL_ERROR,
 					 "Error in AMR octet-aligned <-> bandwidth-efficient mode conversion (target=%s)\n",
-					 conn_dst->end.codec->param.amr_octet_aligned ? "octet-aligned" : "bandwidth-efficient");
+					 dst_codec->param.amr_octet_aligned ? "octet-aligned" : "bandwidth-efficient");
 				msgb_free(msg);
 				return rc;
 			}
 		} else if (rtp_end->rfc5993_hr_convert &&
-			   strcmp(conn_src->end.codec->subtype_name, "GSM-HR-08") == 0) {
+			   strcmp(src_codec->subtype_name, "GSM-HR-08") == 0) {
 			rc = rfc5993_hr_convert(endp, msg);
 			if (rc < 0) {
 				LOGPENDP(endp, DRTP, LOGL_ERROR, "Error while converting to GSM-HR-08\n");
@@ -1251,7 +1247,7 @@ int mgcp_send(struct mgcp_endpoint *endp, int is_rtp, struct osmo_sockaddr *addr
 		return 0;
 	} else if (!trunk->omit_rtcp) {
 		struct osmo_sockaddr rtcp_addr = rtp_end->addr;
-		osmo_sockaddr_set_port(&rtcp_addr.u.sa, rtp_end->rtcp_port);
+		osmo_sockaddr_set_port(&rtcp_addr.u.sa, ntohs(rtp_end->rtcp_port));
 		LOGPENDP(endp, DRTP, LOGL_DEBUG,
 			 "send to %s %s rtp_port:%u rtcp_port:%u\n",
 			 dest_name, osmo_sockaddr_ntop(&rtcp_addr.u.sa, ipbuf),
@@ -1343,13 +1339,13 @@ int mgcp_dispatch_rtp_bridge_cb(struct msgb *msg)
 		 * packets back to their origin. We will use the originating
 		 * address data from the UDP packet header to patch the
 		 * outgoing address in connection on the fly */
-		if (osmo_sockaddr_port(&conn->u.rtp.end.addr.u.sa) == 0) {
-			memcpy(&conn->u.rtp.end.addr, from_addr,
-			       sizeof(conn->u.rtp.end.addr));
+		if (osmo_sockaddr_port(&conn_src->end.addr.u.sa) == 0) {
+			memcpy(&conn_src->end.addr, from_addr,
+			       sizeof(conn_src->end.addr));
 			LOG_CONN_RTP(conn_src, LOGL_NOTICE,
 				     "loopback mode: implicitly using source address (%s:%u) as destination address\n",
 				     osmo_sockaddr_ntop(&from_addr->u.sa, ipbuf),
-				     osmo_sockaddr_port(&conn->u.rtp.end.addr.u.sa));
+				     osmo_sockaddr_port(&conn_src->end.addr.u.sa));
 		}
 		return mgcp_conn_rtp_dispatch_rtp(conn_src, msg);
 	}
@@ -1428,19 +1424,19 @@ int mgcp_dispatch_e1_bridge_cb(struct msgb *msg)
 		 * packets back to their origin. We will use the originating
 		 * address data from the UDP packet header to patch the
 		 * outgoing address in connection on the fly */
-		if (osmo_sockaddr_port(&conn->u.rtp.end.addr.u.sa) == 0) {
-			memcpy(&conn->u.rtp.end.addr, from_addr,
-			       sizeof(conn->u.rtp.end.addr));
+		if (osmo_sockaddr_port(&conn_src->end.addr.u.sa) == 0) {
+			memcpy(&conn_src->end.addr, from_addr,
+			       sizeof(conn_src->end.addr));
 			LOG_CONN_RTP(conn_src, LOGL_NOTICE,
 				     "loopback mode: implicitly using source address (%s:%u) as destination address\n",
 				     osmo_sockaddr_ntop(&from_addr->u.sa, ipbuf),
-				     osmo_sockaddr_port(&conn->u.rtp.end.addr.u.sa));
+				     osmo_sockaddr_port(&conn_src->end.addr.u.sa));
 		}
 		return mgcp_conn_rtp_dispatch_rtp(conn_src, msg);
 	}
 
 	/* Forward to E1 */
-	return mgcp_e1_send_rtp(conn->endp, conn->u.rtp.end.codec, msg);
+	return mgcp_e1_send_rtp(conn->endp, conn_src->end.cset.codec, msg);
 }
 
 /*! cleanup an endpoint when a connection on an RTP bridge endpoint is removed.
@@ -1562,18 +1558,18 @@ static int rx_rtp(struct msgb *msg)
 
 	/* Handle AMR frame format conversion (octet-aligned vs. bandwith-efficient) */
 	if (mc->proto == MGCP_PROTO_RTP
-	    && conn_src->end.codec
-	    && mgcp_codec_amr_align_mode_is_indicated(conn_src->end.codec)) {
+	    && conn_src->end.cset.codec
+	    && mgcp_codec_amr_align_mode_is_indicated(conn_src->end.cset.codec)) {
 		/* Make sure that the incoming AMR frame format matches the frame format that the call agent has
 		 * communicated via SDP when the connection was created/modfied. */
 		int oa = amr_oa_check((char*)msgb_data(msg), msgb_length(msg));
 		if (oa < 0)
 			goto out_free;
-		if (((bool)oa) != conn_src->end.codec->param.amr_octet_aligned) {
+		if (((bool)oa) != conn_src->end.cset.codec->param.amr_octet_aligned) {
 			LOG_CONN_RTP(conn_src, LOGL_NOTICE,
 				     "rx_rtp(%u bytes): Expected RTP AMR octet-aligned=%u but got octet-aligned=%u."
 				     " check the config of your call-agent!\n",
-				     msgb_length(msg), conn_src->end.codec->param.amr_octet_aligned, oa);
+				     msgb_length(msg), conn_src->end.cset.codec->param.amr_octet_aligned, oa);
 			goto out_free;
 		}
 	}
@@ -1629,78 +1625,23 @@ int mgcp_create_bind(const char *source_addr, int port, uint8_t dscp, uint8_t pr
 	return rc;
 }
 
-/* Bind RTP and RTCP port (helper function for mgcp_bind_net_rtp_port()) */
-static int bind_rtp(struct mgcp_config *cfg, const char *source_addr,
-		    struct mgcp_rtp_end *rtp_end, struct mgcp_endpoint *endp)
-{
-	int rc, rtp_fd, rtcp_fd;
-
-	/* NOTE: The port that is used for RTCP is the RTP port incremented by one
-	 * (e.g. RTP-Port = 16000 ==> RTCP-Port = 16001) */
-
-	rc = mgcp_create_bind(source_addr, rtp_end->local_port, cfg->endp_dscp, cfg->endp_priority);
-	if (rc < 0) {
-		LOGPENDP(endp, DRTP, LOGL_ERROR,
-			 "failed to create RTP port: %s:%d\n",
-			 source_addr, rtp_end->local_port);
-		goto cleanup0;
-	}
-	rtp_fd = rc;
-
-	rc = mgcp_create_bind(source_addr, rtp_end->local_port + 1, cfg->endp_dscp, cfg->endp_priority);
-	if (rc < 0) {
-		LOGPENDP(endp, DRTP, LOGL_ERROR,
-			 "failed to create RTCP port: %s:%d\n",
-			 source_addr, rtp_end->local_port + 1);
-		goto cleanup1;
-	}
-	rtcp_fd = rc;
-
-	if (osmo_iofd_register(rtp_end->rtp, rtp_fd) < 0) {
-		LOGPENDP(endp, DRTP, LOGL_ERROR,
-			 "failed to register RTP port %d\n",
-			 rtp_end->local_port);
-		goto cleanup2;
-	}
-
-	if (osmo_iofd_register(rtp_end->rtcp, rtcp_fd) != 0) {
-		LOGPENDP(endp, DRTP, LOGL_ERROR,
-			 "failed to register RTCP port %d\n",
-			 rtp_end->local_port + 1);
-		goto cleanup3;
-	}
-
-	return 0;
-
-cleanup3:
-	osmo_iofd_unregister(rtp_end->rtp);
-cleanup2:
-	close(rtcp_fd);
-cleanup1:
-	close(rtp_fd);
-cleanup0:
-	return -1;
-}
-
 /*! bind RTP port to endpoint/connection.
- *  \param[in] endp endpoint that holds the RTP connection.
- *  \param[in] rtp_port port number to bind on.
  *  \param[in] conn associated RTP connection.
+ *  \param[in] rtp_port port number to bind on.
  *  \returns 0 on success, -1 on ERROR. */
-int mgcp_bind_net_rtp_port(struct mgcp_endpoint *endp, int rtp_port,
-			   struct mgcp_conn_rtp *conn)
+int mgcp_conn_rtp_bind_rtp_ports(struct mgcp_conn_rtp *conn_rtp, int rtp_port)
 {
 	char name[512];
-	struct mgcp_rtp_end *end;
+	struct mgcp_conn *conn = conn_rtp->conn;
+	struct mgcp_config *cfg = conn->endp->trunk->cfg;
+	struct mgcp_rtp_end *end = &conn_rtp->end;
+	int rc, rtp_fd, rtcp_fd;
 
-	snprintf(name, sizeof(name), "%s-%s", conn->conn->name, conn->conn->id);
-	end = &conn->end;
+	snprintf(name, sizeof(name), "%s-%s", conn->name, conn->id);
 
 	if ((end->rtp && osmo_iofd_get_fd(end->rtp) != -1) ||
 	    (end->rtcp && osmo_iofd_get_fd(end->rtcp) != -1)) {
-		LOGPENDP(endp, DRTP, LOGL_ERROR, "%u was already bound on conn:%s\n",
-			 rtp_port, mgcp_conn_dump(conn->conn));
-
+		LOG_CONN_RTP(conn_rtp, LOGL_ERROR, "%u was already bound\n", rtp_port);
 		/* Double bindings should never occour! Since we always allocate
 		 * connections dynamically and free them when they are not
 		 * needed anymore, there must be no previous binding leftover.
@@ -1710,33 +1651,55 @@ int mgcp_bind_net_rtp_port(struct mgcp_endpoint *endp, int rtp_port,
 	}
 
 	end->local_port = rtp_port;
-	end->rtp = osmo_iofd_setup(conn->conn, -1, name, OSMO_IO_FD_MODE_RECVFROM_SENDTO, &rtp_ioops, conn);
+	end->rtp = osmo_iofd_setup(conn, -1, name, OSMO_IO_FD_MODE_RECVFROM_SENDTO, &rtp_ioops, conn_rtp);
 	if (!end->rtp)
-		return -EIO;
+		goto free_iofd_ret;
 	osmo_iofd_set_alloc_info(end->rtp, RTP_BUF_SIZE, 0);
-	end->rtcp = osmo_iofd_setup(conn->conn, -1, name, OSMO_IO_FD_MODE_RECVFROM_SENDTO, &rtp_ioops, conn);
-	if (!end->rtcp) {
-		osmo_iofd_free(end->rtp);
-		end->rtp = NULL;
-		return -EIO;
-	}
+	end->rtcp = osmo_iofd_setup(conn, -1, name, OSMO_IO_FD_MODE_RECVFROM_SENDTO, &rtp_ioops, conn_rtp);
+	if (!end->rtcp)
+		goto free_iofd_ret;
 	osmo_iofd_set_alloc_info(end->rtcp, RTP_BUF_SIZE, 0);
 	osmo_iofd_set_priv_nr(end->rtcp, 1); /* we use priv_nr as identifier for RTCP */
 
-	return bind_rtp(endp->trunk->cfg, conn->end.local_addr, end, endp);
-}
+	/* NOTE: The port that is used for RTCP is the RTP port incremented by one
+	 * (e.g. RTP-Port = 16000 ==> RTCP-Port = 16001) */
 
-/*! free allocated RTP and RTCP ports.
- *  \param[in] end RTP end */
-void mgcp_free_rtp_port(struct mgcp_rtp_end *end)
-{
-	if (end->rtp) {
-		osmo_iofd_free(end->rtp);
-		end->rtp = NULL;
+	rc = mgcp_create_bind(end->local_addr, end->local_port, cfg->endp_dscp, cfg->endp_priority);
+	if (rc < 0) {
+		LOG_CONN_RTP(conn_rtp, LOGL_ERROR, "failed to create RTP port: %s:%d\n", end->local_addr, end->local_port);
+		goto free_iofd_ret;
+	}
+	rtp_fd = rc;
+
+	rc = mgcp_create_bind(end->local_addr, end->local_port + 1, cfg->endp_dscp, cfg->endp_priority);
+	if (rc < 0) {
+		LOG_CONN_RTP(conn_rtp, LOGL_ERROR, "failed to create RTCP port: %s:%d\n", end->local_addr, end->local_port + 1);
+		goto cleanup1;
+	}
+	rtcp_fd = rc;
+
+	if (osmo_iofd_register(end->rtp, rtp_fd) < 0) {
+		LOG_CONN_RTP(conn_rtp, LOGL_ERROR, "failed to register RTP port %d\n", end->local_port);
+		goto cleanup2;
 	}
 
-	if (end->rtcp) {
-		osmo_iofd_free(end->rtcp);
-		end->rtcp = NULL;
+	if (osmo_iofd_register(end->rtcp, rtcp_fd) != 0) {
+		LOG_CONN_RTP(conn_rtp, LOGL_ERROR, "failed to register RTCP port %d\n", end->local_port + 1);
+		goto cleanup3;
 	}
+
+	return 0;
+
+cleanup3:
+	osmo_iofd_unregister(end->rtp);
+cleanup2:
+	close(rtcp_fd);
+cleanup1:
+	close(rtp_fd);
+free_iofd_ret:
+	osmo_iofd_free(end->rtcp);
+	end->rtcp = NULL;
+	osmo_iofd_free(end->rtp);
+	end->rtp = NULL;
+	return -EIO;
 }
