@@ -41,7 +41,6 @@
 #include <osmocom/mgcp/debug.h>
 #include <osmocom/mgcp/mgcp_endp.h>
 #include <osmocom/mgcp/mgcp_trunk.h>
-#include <osmocom/mgcp/mgcp_ctrl.h>
 
 #include <osmocom/core/application.h>
 #include <osmocom/core/msgb.h>
@@ -51,7 +50,7 @@
 #include <osmocom/core/rate_ctr.h>
 #include <osmocom/core/logging.h>
 #include <osmocom/core/socket.h>
-
+#include <osmocom/ctrl/control_if.h>
 #include <osmocom/ctrl/control_vty.h>
 
 #include <osmocom/vty/telnet_interface.h>
@@ -83,7 +82,7 @@ static int daemonize = 0;
 
 const char *osmomgw_copyright =
 	"Copyright (C) 2009-2010 Holger Freyther and On-Waves\r\n"
-	"Copyright (C) 2017 by sysmocom s.f.m.c. GmbH <info@sysmocom.de>\r\n"
+	"Copyright (C) 2017-2022 by sysmocom s.f.m.c. GmbH\r\n"
 	"Contributions by Pablo Neira Ayuso, Jacob Erlbeck, Neels Hofmeyr\r\n"
 	"Philipp Maier\r\n\r\n"
 	"License AGPLv3+: GNU AGPL version 3 or later <http://gnu.org/licenses/agpl-3.0.html>\r\n"
@@ -95,7 +94,7 @@ static char *config_file = "osmo-mgw.cfg";
 /* used by msgb and mgcp */
 void *tall_mgw_ctx = NULL;
 
-static void print_help()
+static void print_help(void)
 {
 	printf("Some useful options:\n");
 	printf(" -h --help is printing this text.\n");
@@ -281,6 +280,31 @@ int mgcp_vty_go_parent(struct vty *vty)
 	return vty->node;
 }
 
+static void signal_handler(int signum)
+{
+	fprintf(stdout, "signal %u received\n", signum);
+
+	switch (signum) {
+	case SIGABRT:
+		/* in case of abort, we want to obtain a talloc report and
+		 * then run default SIGABRT handler, who will generate coredump
+		 * and abort the process. abort() should do this for us after we
+		 * return, but program wouldn't exit if an external SIGABRT is
+		 * received.
+		 */
+		talloc_report(tall_vty_ctx, stderr);
+		talloc_report_full(tall_mgw_ctx, stderr);
+		signal(SIGABRT, SIG_DFL);
+		raise(SIGABRT);
+		break;
+	case SIGUSR1:
+		talloc_report(tall_vty_ctx, stderr);
+		talloc_report_full(tall_mgw_ctx, stderr);
+		break;
+	default:
+		break;
+	}
+}
 
 static struct vty_app_info vty_info = {
 	.name 		= "OsmoMGW",
@@ -295,13 +319,22 @@ static const struct log_info_cat log_categories[] = {
 		  .name = "DRTP",
 		  .description = "RTP stream handling",
 		  .color = "\033[1;30m",
-		  .enabled = 1,.loglevel = LOGL_NOTICE,
+		  .enabled = 1,
+		  .loglevel = LOGL_NOTICE,
 	},
 	[DE1] = {
 		  .name = "DE1",
 		  .description = "E1 line handling",
 		  .color = "\033[1;31m",
-		  .enabled = 1,.loglevel = LOGL_NOTICE,
+		  .enabled = 1,
+		  .loglevel = LOGL_NOTICE,
+	},
+	[DOSMUX] = {
+		  .name = "DOSMUX",
+		  .description = "Osmux (Osmocom RTP multiplexing)",
+		  .color = "\033[1;32m",
+		  .enabled = 1,
+		  .loglevel = LOGL_NOTICE,
 	},
 };
 
@@ -320,6 +353,8 @@ int main(int argc, char **argv)
 
 	msgb_talloc_ctx_init(tall_mgw_ctx, 0);
 
+	signal(SIGABRT, &signal_handler);
+	signal(SIGUSR1, &signal_handler);
 	osmo_init_ignore_signals();
 	osmo_init_logging2(tall_mgw_ctx, &log_info);
 	libosmo_abis_init(tall_mgw_ctx);
@@ -352,12 +387,11 @@ int main(int argc, char **argv)
 		return rc;
 
 	/* start telnet after reading config for vty_get_bind_addr() */
-	rc = telnet_init_dynif(tall_mgw_ctx, NULL,
-			       vty_get_bind_addr(), OSMO_VTY_PORT_MGW);
+	rc = telnet_init_default(tall_mgw_ctx, NULL, OSMO_VTY_PORT_MGW);
 	if (rc < 0)
 		return rc;
 
-	cfg->ctrl = mgw_ctrl_interface_setup(cfg, ctrl_vty_get_bind_addr(), OSMO_CTRL_PORT_MGW);
+	cfg->ctrl = ctrl_interface_setup(cfg, OSMO_CTRL_PORT_MGW, NULL);
 	if (!cfg->ctrl) {
 		fprintf(stderr, "Failed to init the control interface on %s:%u. Exiting\n",
 			ctrl_vty_get_bind_addr(), OSMO_CTRL_PORT_MGW);
